Customizing DDL — SQLAlchemy 1.4 Documentation

Release: 1.4.48 | Release Date: April 30, 2023

# SQLAlchemy 1.4 Documentation

# Customizing DDL

In the preceding sections we’ve discussed a variety of schema constructs including `Table`, `ForeignKeyConstraint`, `CheckConstraint`, and `Sequence`. Throughout, we’ve relied upon the `create()` and `create_all()` methods of `Table` and `MetaData` in order to issue data definition language (DDL) for all constructs. When issued, a pre-determined order of operations is invoked, and DDL to create each table is created unconditionally including all constraints and other objects associated with it. For more complex scenarios where database-specific DDL is required, SQLAlchemy offers two techniques which can be used to add any DDL based on any condition, either accompanying the standard generation of tables or by itself.

## Custom DDL

Custom DDL phrases are most easily achieved using the `DDL` construct. This construct works like all the other DDL elements except it accepts a string which is the text to be emitted:

    event.listen(
        metadata,
        "after_create",
        DDL(
            "ALTER TABLE users ADD CONSTRAINT "
            "cst_user_name_length "
            " CHECK (length(user_name) >= 8)"
        ),
    )

A more comprehensive method of creating libraries of DDL constructs is to use custom compilation - see Custom SQL Constructs and Compilation Extension for details.

## Controlling DDL Sequences

The `DDL` construct introduced previously also has the ability to be invoked conditionally based on inspection of the database. This feature is available using the `DDLElement.execute_if()` method. For example, if we wanted to create a trigger but only on the PostgreSQL backend, we could invoke this as:

    mytable = Table(
        "mytable",
        metadata,
        Column("id", Integer, primary_key=True),
        Column("data", String(50)),
    )

    func = DDL(
        "CREATE FUNCTION my_func() "
        "RETURNS TRIGGER AS $$ "
        "BEGIN "
        "NEW.data := 'ins'; "
        "RETURN NEW; "
        "END; $$ LANGUAGE PLPGSQL"
    )

    trigger = DDL(
        "CREATE TRIGGER dt_ins BEFORE INSERT ON mytable "
        "FOR EACH ROW EXECUTE PROCEDURE my_func();"
    )

    event.listen(mytable, "after_create", func.execute_if(dialect="postgresql"))

    event.listen(mytable, "after_create", trigger.execute_if(dialect="postgresql"))

The `DDLElement.execute_if.dialect` keyword also accepts a tuple of string dialect names:

    event.listen(
        mytable, "after_create", trigger.execute_if(dialect=("postgresql", "mysql"))
    )
    event.listen(
        mytable, "before_drop", trigger.execute_if(dialect=("postgresql", "mysql"))
    )

The `DDLElement.execute_if()` method can also work against a callable function that will receive the database connection in use. In the example below, we use this to conditionally create a CHECK constraint, first looking within the PostgreSQL catalogs to see if it exists:

    def should_create(ddl, target, connection, **kw):
        row = connection.execute(
            "select conname from pg_constraint where conname='%s'" % ddl.element.name
        ).scalar()
        return not bool(row)

    def should_drop(ddl, target, connection, **kw):
        return not should_create(ddl, target, connection, **kw)

    event.listen(
        users,
        "after_create",
        DDL(
            "ALTER TABLE users ADD CONSTRAINT "
            "cst_user_name_length CHECK (length(user_name) >= 8)"
        ).execute_if(callable_=should_create),
    )
    event.listen(
        users,
        "before_drop",
        DDL("ALTER TABLE users DROP CONSTRAINT cst_user_name_length").execute_if(
            callable_=should_drop
        ),
    )

    sqlusers.create(engine)
    CREATE TABLE users (
        user_id SERIAL NOT NULL,
        user_name VARCHAR(40) NOT NULL,
        PRIMARY KEY (user_id)
    )

    select conname from pg_constraint where conname='cst_user_name_length'
    ALTER TABLE users ADD CONSTRAINT cst_user_name_length  CHECK (length(user_name) >= 8)

    sqlusers.drop(engine)
    select conname from pg_constraint where conname='cst_user_name_length'
    ALTER TABLE users DROP CONSTRAINT cst_user_name_length
    DROP TABLE users

## Using the built-in DDLElement Classes

The `sqlalchemy.schema` package contains SQL expression constructs that provide DDL expressions. For example, to produce a `CREATE TABLE` statement:

    from sqlalchemy.schema import CreateTable
    with engine.connect() as conn:
    sql    conn.execute(CreateTable(mytable))
    CREATE TABLE mytable (
        col1 INTEGER,
        col2 INTEGER,
        col3 INTEGER,
        col4 INTEGER,
        col5 INTEGER,
        col6 INTEGER
    )

Above, the `CreateTable` construct works like any other expression construct (such as `select()`, `table.insert()`, etc.). All of SQLAlchemy’s DDL oriented constructs are subclasses of the `DDLElement` base class; this is the base of all the objects corresponding to CREATE and DROP as well as ALTER, not only in SQLAlchemy but in Alembic Migrations as well. A full reference of available constructs is in DDL Expression Constructs API.

User-defined DDL constructs may also be created as subclasses of `DDLElement` itself. The documentation in Custom SQL Constructs and Compilation Extension has several examples of this.

The event-driven DDL system described in the previous section Controlling DDL Sequences is available with other `DDLElement` objects as well. However, when dealing with the built-in constructs such as `CreateIndex`, `CreateSequence`, etc, the event system is of limited use, as methods like `Table.create()` and `MetaData.create_all()` will invoke these constructs unconditionally. In a future SQLAlchemy release, the DDL event system including conditional execution will taken into account for built-in constructs that currently invoke in all cases.

We can illustrate an event-driven example with the `AddConstraint` and `DropConstraint` constructs, as the event-driven system will work for CHECK and UNIQUE constraints, using these as we did in our previous example of `DDLElement.execute_if()`:

    def should_create(ddl, target, connection, **kw):
        row = connection.execute(
            "select conname from pg_constraint where conname='%s'" % ddl.element.name
        ).scalar()
        return not bool(row)

    def should_drop(ddl, target, connection, **kw):
        return not should_create(ddl, target, connection, **kw)

    event.listen(
        users, "after_create", AddConstraint(constraint).execute_if(callable_=should_create)
    )
    event.listen(
        users, "before_drop", DropConstraint(constraint).execute_if(callable_=should_drop)
    )

    sqlusers.create(engine)
    CREATE TABLE users (
        user_id SERIAL NOT NULL,
        user_name VARCHAR(40) NOT NULL,
        PRIMARY KEY (user_id)
    )

    select conname from pg_constraint where conname='cst_user_name_length'
    ALTER TABLE users ADD CONSTRAINT cst_user_name_length  CHECK (length(user_name) >= 8)

    sqlusers.drop(engine)
    select conname from pg_constraint where conname='cst_user_name_length'
    ALTER TABLE users DROP CONSTRAINT cst_user_name_length
    DROP TABLE users

While the above example is against the built-in `AddConstraint` and `DropConstraint` objects, the main usefulness of DDL events for now remains focused on the use of the `DDL` construct itself, as well as with user-defined subclasses of `DDLElement` that aren’t already part of the `MetaData.create_all()`, `Table.create()`, and corresponding “drop” processes.

## DDL Expression Constructs API

Object Name | Description
--- | ---
_CreateDropBase | Base class for DDL constructs that represent CREATE and DROP or equivalents.
AddConstraint | Represent an ALTER TABLE ADD CONSTRAINT statement.
CreateColumn | Represent a `Column` as rendered in a CREATE TABLE statement, via the `CreateTable` construct.
CreateIndex | Represent a CREATE INDEX statement.
CreateSchema | Represent a CREATE SCHEMA statement.
CreateSequence | Represent a CREATE SEQUENCE statement.
CreateTable | Represent a CREATE TABLE statement.
DDL | A literal DDL statement.
DDLElement | Base class for DDL expression constructs.
DropConstraint | Represent an ALTER TABLE DROP CONSTRAINT statement.
DropIndex | Represent a DROP INDEX statement.
DropSchema | Represent a DROP SCHEMA statement.
DropSequence | Represent a DROP SEQUENCE statement.
DropTable | Represent a DROP TABLE statement.
sort_tables(tables[, skip_fn, extra_dependencies]) | Sort a collection of `Table` objects based on dependency.
sort_tables_and_constraints(tables[, filter_fn, extra_dependencies, _warn_for_cycles]) | Sort a collection of `Table` / `ForeignKeyConstraint` objects.

function sqlalchemy.schema.sort_tables(tables, skip_fn=None, extra_dependencies=None)

  Sort a collection of `Table` objects based on dependency.

  This is a dependency-ordered sort which will emit `Table` objects such that they will follow their dependent `Table` objects. Tables are dependent on another based on the presence of `ForeignKeyConstraint` objects as well as explicit dependencies added by `Table.add_is_dependent_on()`.

  Warning

  The `sort_tables()` function cannot by itself accommodate automatic resolution of dependency cycles between tables, which are usually caused by mutually dependent foreign key constraints. When these cycles are detected, the foreign keys of these tables are omitted from consideration in the sort. A warning is emitted when this condition occurs, which will be an exception raise in a future release. Tables which are not part of the cycle will still be returned in dependency order.

  To resolve these cycles, the `ForeignKeyConstraint.use_alter` parameter may be applied to those constraints which create a cycle. Alternatively, the `sort_tables_and_constraints()` function will automatically return foreign key constraints in a separate collection when cycles are detected so that they may be applied to a schema separately.

  Changed in version 1.3.17: - a warning is emitted when `sort_tables()` cannot perform a proper sort due to cyclical dependencies. This will be an exception in a future release. Additionally, the sort will continue to return other tables not involved in the cycle in dependency order which was not the case previously.

  Parameters:

    - tables – a sequence of `Table` objects.

    - skip_fn – optional callable which will be passed a `ForeignKey` object; if it returns True, this constraint will not be considered as a dependency. Note this is different from the same parameter in `sort_tables_and_constraints()`, which is instead passed the owning `ForeignKeyConstraint` object.

    - extra_dependencies – a sequence of 2-tuples of tables which will also be considered as dependent on each other.

  See also

  `sort_tables_and_constraints()`

  `MetaData.sorted_tables` - uses this function to sort

function sqlalchemy.schema.sort_tables_and_constraints(tables, filter_fn=None, extra_dependencies=None, _warn_for_cycles=False)

  Sort a collection of `Table` / `ForeignKeyConstraint` objects.

  This is a dependency-ordered sort which will emit tuples of `(Table, [ForeignKeyConstraint, ...])` such that each `Table` follows its dependent `Table` objects. Remaining `ForeignKeyConstraint` objects that are separate due to dependency rules not satisfied by the sort are emitted afterwards as `(None, [ForeignKeyConstraint ...])`.

  Tables are dependent on another based on the presence of `ForeignKeyConstraint` objects, explicit dependencies added by `Table.add_is_dependent_on()`, as well as dependencies stated here using the `sort_tables_and_constraints.skip_fn` and/or `sort_tables_and_constraints.extra_dependencies` parameters.

  Parameters:

    - tables – a sequence of `Table` objects.

    - filter_fn – optional callable which will be passed a `ForeignKeyConstraint` object, and returns a value based on whether this constraint should definitely be included or excluded as an inline constraint, or neither. If it returns False, the constraint will definitely be included as a dependency that cannot be subject to ALTER; if True, it will only be included as an ALTER result at the end. Returning None means the constraint is included in the table-based result unless it is detected as part of a dependency cycle.

    - extra_dependencies – a sequence of 2-tuples of tables which will also be considered as dependent on each other.

  New in version 1.0.0.

  See also

  `sort_tables()`

class sqlalchemy.schema.DDLElement

  Base class for DDL expression constructs.

  This class is the base for the general purpose `DDL` class, as well as the various create/drop clause constructs such as `CreateTable`, `DropTable`, `AddConstraint`, etc.

  `DDLElement` integrates closely with SQLAlchemy events, introduced in Events. An instance of one is itself an event receiving callable:

      event.listen(
          users,
          'after_create',
          AddConstraint(constraint).execute_if(dialect='postgresql')
      )

  See also

  `DDL`

  `DDLEvents`

  Events

  Controlling DDL Sequences

  Members

  __call__(), against(), bind, callable_, dialect, execute(), execute_if(), on, target

  Class signature

  class `sqlalchemy.schema.DDLElement` (`sqlalchemy.sql.roles.DDLRole`, `sqlalchemy.sql.expression.Executable`, `sqlalchemy.schema._DDLCompiles`)

  method `sqlalchemy.schema.DDLElement.` __call__(target, bind, **kw)

    Execute the DDL as a ddl_listener.

  method `sqlalchemy.schema.DDLElement.` against(target)

    Return a copy of this `DDLElement` which will include the given target.

    This essentially applies the given item to the `.target` attribute of the returned `DDLElement` object. This target is then usable by event handlers and compilation routines in order to provide services such as tokenization of a DDL string in terms of a particular `Table`.

    When a `DDLElement` object is established as an event handler for the `DDLEvents.before_create()` or `DDLEvents.after_create()` events, and the event then occurs for a given target such as a `Constraint` or `Table`, that target is established with a copy of the `DDLElement` object using this method, which then proceeds to the `DDLElement.execute()` method in order to invoke the actual DDL instruction.

    Parameters:

      target – a `SchemaItem` that will be the subject of a DDL operation.

    Returns:

      a copy of this `DDLElement` with the `.target` attribute assigned to the given `SchemaItem`.

    See also

    `DDL` - uses tokenization against the “target” when processing the DDL string.

  attribute `sqlalchemy.schema.DDLElement.` bind
  attribute `sqlalchemy.schema.DDLElement.` callable_ = None
  attribute `sqlalchemy.schema.DDLElement.` dialect = None
  method `sqlalchemy.schema.DDLElement.` execute(bind=None, target=None)

    Execute this DDL immediately.

    Deprecated since version 1.4: The `DDLElement.execute()` method is considered legacy as of the 1.x series of SQLAlchemy and will be removed in 2.0. All statement execution in SQLAlchemy 2.0 is performed by the `Connection.execute()` method of `Connection`, or in the ORM by the `Session.execute()` method of `Session`. (Background on SQLAlchemy 2.0 at: Migrating to SQLAlchemy 2.0)

    Executes the DDL statement in isolation using the supplied `Connectable` or `Connectable` assigned to the `.bind` property, if not supplied. If the DDL has a conditional `on` criteria, it will be invoked with None as the event.

    Parameters:

      - bind – Optional, an `Engine` or `Connection`. If not supplied, a valid `Connectable` must be present in the `.bind` property.

      - target – Optional, defaults to None. The target `SchemaItem` for the execute call. This is equivalent to passing the `SchemaItem` to the `DDLElement.against()` method and then invoking `DDLElement.execute()` upon the resulting `DDLElement` object. See `DDLElement.against()` for further detail.

  method `sqlalchemy.schema.DDLElement.` execute_if(dialect=None, callable_=None, state=None)

    Return a callable that will execute this `DDLElement` conditionally within an event handler.

    Used to provide a wrapper for event listening:

        event.listen(
                    metadata,
                    'before_create',
                    DDL("my_ddl").execute_if(dialect='postgresql')
                )

    Parameters:

      - dialect –

        May be a string or tuple of strings. If a string, it will be compared to the name of the executing database dialect:

            DDL('something').execute_if(dialect='postgresql')

        If a tuple, specifies multiple dialect names:

            DDL('something').execute_if(dialect=('postgresql', 'mysql'))

      - callable_ –

        A callable, which will be invoked with four positional arguments as well as optional keyword arguments:

        > ddl:

        >   This DDL element.

        > target:

        >   The `Table` or `MetaData` object which is the target of this event. May be None if the DDL is executed explicitly.

        > bind:

        >   The `Connection` being used for DDL execution

        > tables:

        >   Optional keyword argument - a list of Table objects which are to be created/ dropped within a MetaData.create_all() or drop_all() method call.

        > state:

        >   Optional keyword argument - will be the `state` argument passed to this function.

        > checkfirst:

        >   Keyword argument, will be True if the ‘checkfirst’ flag was set during the call to `create()`, `create_all()`, `drop()`, `drop_all()`.

        If the callable returns a True value, the DDL statement will be executed.

      - state – any value which will be passed to the callable_ as the `state` keyword argument.

    See also

    `DDLEvents`

    Events

  attribute `sqlalchemy.schema.DDLElement.` on = None
  attribute `sqlalchemy.schema.DDLElement.` target = None

class sqlalchemy.schema.DDL(statement, context=None, bind=None)

  A literal DDL statement.

  Specifies literal SQL DDL to be executed by the database. DDL objects function as DDL event listeners, and can be subscribed to those events listed in `DDLEvents`, using either `Table` or `MetaData` objects as targets. Basic templating support allows a single DDL instance to handle repetitive tasks for multiple tables.

  Examples:

      from sqlalchemy import event, DDL

      tbl = Table('users', metadata, Column('uid', Integer))
      event.listen(tbl, 'before_create', DDL('DROP TRIGGER users_trigger'))

      spow = DDL('ALTER TABLE %(table)s SET secretpowers TRUE')
      event.listen(tbl, 'after_create', spow.execute_if(dialect='somedb'))

      drop_spow = DDL('ALTER TABLE users SET secretpowers FALSE')
      connection.execute(drop_spow)

  When operating on Table events, the following `statement` string substitutions are available:

      %(table)s  - the Table name, with any required quoting applied
      %(schema)s - the schema name, with any required quoting applied
      %(fullname)s - the Table name including schema, quoted if needed

  The DDL’s “context”, if any, will be combined with the standard substitutions noted above. Keys present in the context will override the standard substitutions.

  Members

  __init__()

  Class signature

  class `sqlalchemy.schema.DDL` (`sqlalchemy.schema.DDLElement`)

  method `sqlalchemy.schema.DDL.` __init__(statement, context=None, bind=None)

    Create a DDL statement.

    Parameters:

      - statement –

        A string or unicode string to be executed. Statements will be processed with Python’s string formatting operator using a fixed set of string substitutions, as well as additional substitutions provided by the optional `DDL.context` parameter.

        A literal ‘%’ in a statement must be escaped as ‘%%’.

        SQL bind parameters are not available in DDL statements.

      - context – Optional dictionary, defaults to None. These values will be available for use in string substitutions on the DDL statement.

      - bind –

        Optional. A `Connectable`, used by default when `execute()` is invoked without a bind argument.

        Deprecated since version 1.4: The `DDL.bind` argument is deprecated and will be removed in SQLAlchemy 2.0.

    See also

    `DDLEvents`

    Events

class sqlalchemy.schema._CreateDropBase(element, bind=None, if_exists=False, if_not_exists=False, _legacy_bind=None)

  Base class for DDL constructs that represent CREATE and DROP or equivalents.

  The common theme of _CreateDropBase is a single `element` attribute which refers to the element to be created or dropped.

  Class signature

  class `sqlalchemy.schema._CreateDropBase` (`sqlalchemy.schema.DDLElement`)

class sqlalchemy.schema.CreateTable(element, bind=None, include_foreign_key_constraints=None, if_not_exists=False)

  Represent a CREATE TABLE statement.

  Members

  __init__()

  Class signature

  class `sqlalchemy.schema.CreateTable` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.CreateTable.` __init__(element, bind=None, include_foreign_key_constraints=None, if_not_exists=False)

    Create a `CreateTable` construct.

    Parameters:

      - element – a `Table` that’s the subject of the CREATE

      - on – See the description for ‘on’ in `DDL`.

      - bind – See the description for ‘bind’ in `DDL`.

    Deprecated since version 1.4: The `CreateTable.bind` argument is deprecated and will be removed in SQLAlchemy 2.0.

    Parameters:

      - include_foreign_key_constraints –

        optional sequence of `ForeignKeyConstraint` objects that will be included inline within the CREATE construct; if omitted, all foreign key constraints that do not specify use_alter=True are included.

        New in version 1.0.0.

      - if_not_exists –

        if True, an IF NOT EXISTS operator will be applied to the construct.

        New in version 1.4.0b2.

class sqlalchemy.schema.DropTable(element, bind=None, if_exists=False)

  Represent a DROP TABLE statement.

  Members

  __init__()

  Class signature

  class `sqlalchemy.schema.DropTable` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.DropTable.` __init__(element, bind=None, if_exists=False)

    Create a `DropTable` construct.

    Parameters:

      - element – a `Table` that’s the subject of the DROP.

      - on – See the description for ‘on’ in `DDL`.

      - bind – See the description for ‘bind’ in `DDL`.

    Deprecated since version 1.4: The `DropTable.bind` argument is deprecated and will be removed in SQLAlchemy 2.0.

    Parameters:

      if_exists –

      if True, an IF EXISTS operator will be applied to the construct.

      New in version 1.4.0b2.

class sqlalchemy.schema.CreateColumn(element)

  Represent a `Column` as rendered in a CREATE TABLE statement, via the `CreateTable` construct.

  This is provided to support custom column DDL within the generation of CREATE TABLE statements, by using the compiler extension documented in Custom SQL Constructs and Compilation Extension to extend `CreateColumn`.

  Typical integration is to examine the incoming `Column` object, and to redirect compilation if a particular flag or condition is found:

      from sqlalchemy import schema
      from sqlalchemy.ext.compiler import compiles

      @compiles(schema.CreateColumn)
      def compile(element, compiler, **kw):
          column = element.element

          if "special" not in column.info:
              return compiler.visit_create_column(element, **kw)

          text = "%s SPECIAL DIRECTIVE %s" % (
                  column.name,
                  compiler.type_compiler.process(column.type)
              )
          default = compiler.get_column_default_string(column)
          if default is not None:
              text += " DEFAULT " + default

          if not column.nullable:
              text += " NOT NULL"

          if column.constraints:
              text += " ".join(
                          compiler.process(const)
                          for const in column.constraints)
          return text

  The above construct can be applied to a `Table` as follows:

      from sqlalchemy import Table, Metadata, Column, Integer, String
      from sqlalchemy import schema

      metadata = MetaData()

      table = Table('mytable', MetaData(),
              Column('x', Integer, info={"special":True}, primary_key=True),
              Column('y', String(50)),
              Column('z', String(20), info={"special":True})
          )

      metadata.create_all(conn)

  Above, the directives we’ve added to the `Column.info` collection will be detected by our custom compilation scheme:

      CREATE TABLE mytable (
              x SPECIAL DIRECTIVE INTEGER NOT NULL,
              y VARCHAR(50),
              z SPECIAL DIRECTIVE VARCHAR(20),
          PRIMARY KEY (x)
      )

  The `CreateColumn` construct can also be used to skip certain columns when producing a `CREATE TABLE`. This is accomplished by creating a compilation rule that conditionally returns `None`. This is essentially how to produce the same effect as using the `system=True` argument on `Column`, which marks a column as an implicitly-present “system” column.

  For example, suppose we wish to produce a `Table` which skips rendering of the PostgreSQL `xmin` column against the PostgreSQL backend, but on other backends does render it, in anticipation of a triggered rule. A conditional compilation rule could skip this name only on PostgreSQL:

      from sqlalchemy.schema import CreateColumn

      @compiles(CreateColumn, "postgresql")
      def skip_xmin(element, compiler, **kw):
          if element.element.name == 'xmin':
              return None
          else:
              return compiler.visit_create_column(element, **kw)

      my_table = Table('mytable', metadata,
                  Column('id', Integer, primary_key=True),
                  Column('xmin', Integer)
              )

  Above, a `CreateTable` construct will generate a `CREATE TABLE` which only includes the `id` column in the string; the `xmin` column will be omitted, but only against the PostgreSQL backend.

  Class signature

  class `sqlalchemy.schema.CreateColumn` (`sqlalchemy.schema._DDLCompiles`)

class sqlalchemy.schema.CreateSequence(element, bind=None, if_exists=False, if_not_exists=False, _legacy_bind=None)

  Represent a CREATE SEQUENCE statement.

  Class signature

  class `sqlalchemy.schema.CreateSequence` (`sqlalchemy.schema._CreateDropBase`)

class sqlalchemy.schema.DropSequence(element, bind=None, if_exists=False, if_not_exists=False, _legacy_bind=None)

  Represent a DROP SEQUENCE statement.

  Class signature

  class `sqlalchemy.schema.DropSequence` (`sqlalchemy.schema._CreateDropBase`)

class sqlalchemy.schema.CreateIndex(element, bind=None, if_not_exists=False)

  Represent a CREATE INDEX statement.

  Members

  __init__()

  Class signature

  class `sqlalchemy.schema.CreateIndex` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.CreateIndex.` __init__(element, bind=None, if_not_exists=False)

    Create a `Createindex` construct.

    Parameters:

      - element – a `Index` that’s the subject of the CREATE.

      - on – See the description for ‘on’ in `DDL`.

      - bind – See the description for ‘bind’ in `DDL`.

    Deprecated since version 1.4: The `CreateIndex.bind` argument is deprecated and will be removed in SQLAlchemy 2.0.

    Parameters:

      if_not_exists –

      if True, an IF NOT EXISTS operator will be applied to the construct.

      New in version 1.4.0b2.

class sqlalchemy.schema.DropIndex(element, bind=None, if_exists=False)

  Represent a DROP INDEX statement.

  Members

  __init__()

  Class signature

  class `sqlalchemy.schema.DropIndex` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.DropIndex.` __init__(element, bind=None, if_exists=False)

    Create a `DropIndex` construct.

    Parameters:

      - element – a `Index` that’s the subject of the DROP.

      - on – See the description for ‘on’ in `DDL`.

      - bind – See the description for ‘bind’ in `DDL`.

    Deprecated since version 1.4: The `DropIndex.bind` argument is deprecated and will be removed in SQLAlchemy 2.0.

    Parameters:

      if_exists –

      if True, an IF EXISTS operator will be applied to the construct.

      New in version 1.4.0b2.

class sqlalchemy.schema.AddConstraint(element, *args, **kw)

  Represent an ALTER TABLE ADD CONSTRAINT statement.

  Class signature

  class `sqlalchemy.schema.AddConstraint` (`sqlalchemy.schema._CreateDropBase`)

class sqlalchemy.schema.DropConstraint(element, cascade=False, **kw)

  Represent an ALTER TABLE DROP CONSTRAINT statement.

  Class signature

  class `sqlalchemy.schema.DropConstraint` (`sqlalchemy.schema._CreateDropBase`)

class sqlalchemy.schema.CreateSchema(name, quote=None, **kw)

  Represent a CREATE SCHEMA statement.

  The argument here is the string name of the schema.

  Members

  __init__(), stringify_dialect

  Class signature

  class `sqlalchemy.schema.CreateSchema` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.CreateSchema.` __init__(name, quote=None, **kw)

    Create a new `CreateSchema` construct.

  attribute `sqlalchemy.schema.CreateSchema.` stringify_dialect = 'default'

class sqlalchemy.schema.DropSchema(name, quote=None, cascade=False, **kw)

  Represent a DROP SCHEMA statement.

  The argument here is the string name of the schema.

  Members

  __init__(), stringify_dialect

  Class signature

  class `sqlalchemy.schema.DropSchema` (`sqlalchemy.schema._CreateDropBase`)

  method `sqlalchemy.schema.DropSchema.` __init__(name, quote=None, cascade=False, **kw)

    Create a new `DropSchema` construct.

  attribute `sqlalchemy.schema.DropSchema.` stringify_dialect = 'default'